@@ -1,7 +1,6 @@
 import getpass
 import hashlib
 import json
-import mimetypes
 import os
 import pkgutil
 import re
@@ -9,20 +8,23 @@ import sys
 import time
 import typing as t
 import uuid
+from io import BytesIO
 from itertools import chain
 from os.path import basename
 from os.path import join
+from zlib import adler32
 
 from .._internal import _log
+from ..exceptions import NotFound
 from ..http import parse_cookie
 from ..security import gen_salt
+from ..utils import send_file
 from ..wrappers.request import Request
 from ..wrappers.response import Response
 from .console import Console
-from .tbtools import Frame
-from .tbtools import get_current_traceback
+from .tbtools import DebugFrameSummary
+from .tbtools import DebugTraceback
 from .tbtools import render_console_html
-from .tbtools import Traceback
 
 if t.TYPE_CHECKING:
     from _typeshed.wsgi import StartResponse
@@ -126,6 +128,9 @@ class _ConsoleFrame:
         self.console = Console(namespace)
         self.id = 0
 
+    def eval(self, code: str) -> t.Any:
+        return self.console.eval(code)
+
 
 def get_pin_and_cookie_name(
     app: "WSGIApplication",
@@ -226,7 +231,7 @@ class DebuggedApplication:
     :param app: the WSGI application to run debugged.
     :param evalex: enable exception evaluation feature (interactive
                    debugging).  This requires a non-forking server.
-    :param request_key: The key that points to the request object in ths
+    :param request_key: The key that points to the request object in this
                         environment.  This parameter is ignored in current
                         versions.
     :param console_path: the URL for a general purpose console.
@@ -258,8 +263,7 @@ class DebuggedApplication:
             console_init_func = None
         self.app = app
         self.evalex = evalex
-        self.frames: t.Dict[int, t.Union[Frame, _ConsoleFrame]] = {}
-        self.tracebacks: t.Dict[int, Traceback] = {}
+        self.frames: t.Dict[int, t.Union[DebugFrameSummary, _ConsoleFrame]] = {}
         self.request_key = request_key
         self.console_path = console_path
         self.console_init_func = console_init_func
@@ -308,28 +312,25 @@ class DebuggedApplication:
             yield from app_iter
             if hasattr(app_iter, "close"):
                 app_iter.close()  # type: ignore
-        except Exception:
+        except Exception as e:
             if hasattr(app_iter, "close"):
                 app_iter.close()  # type: ignore
-            traceback = get_current_traceback(
-                skip=1,
-                show_hidden_frames=self.show_hidden_frames,
-                ignore_system_exceptions=True,
+
+            tb = DebugTraceback(e, skip=1, hide=not self.show_hidden_frames)
+
+            for frame in tb.all_frames:
+                self.frames[id(frame)] = frame
+
+            is_trusted = bool(self.check_pin_trust(environ))
+            html = tb.render_debugger_html(
+                evalex=self.evalex,
+                secret=self.secret,
+                evalex_trusted=is_trusted,
             )
-            for frame in traceback.frames:
-                self.frames[frame.id] = frame
-            self.tracebacks[traceback.id] = traceback
+            response = Response(html, status=500, mimetype="text/html")
 
             try:
-                start_response(
-                    "500 INTERNAL SERVER ERROR",
-                    [
-                        ("Content-Type", "text/html; charset=utf-8"),
-                        # Disable Chrome's XSS protection, the debug
-                        # output can cause false-positives.
-                        ("X-XSS-Protection", "0"),
-                    ],
-                )
+                yield from response(environ, start_response)
             except Exception:
                 # if we end up here there has been output but an error
                 # occurred.  in that situation we can do nothing fancy any
@@ -340,19 +341,17 @@ class DebuggedApplication:
                     "response at a point where response headers were already "
                     "sent.\n"
                 )
-            else:
-                is_trusted = bool(self.check_pin_trust(environ))
-                yield traceback.render_full(
-                    evalex=self.evalex, evalex_trusted=is_trusted, secret=self.secret
-                ).encode("utf-8", "replace")
 
-            traceback.log(environ["wsgi.errors"])
+            environ["wsgi.errors"].write("".join(tb.render_traceback_text()))
 
     def execute_command(
-        self, request: Request, command: str, frame: t.Union[Frame, _ConsoleFrame]
+        self,
+        request: Request,
+        command: str,
+        frame: t.Union[DebugFrameSummary, _ConsoleFrame],
     ) -> Response:
         """Execute a command in a console."""
-        return Response(frame.console.eval(command), mimetype="text/html")
+        return Response(frame.eval(command), mimetype="text/html")
 
     def display_console(self, request: Request) -> Response:
         """Display a standalone shell."""
@@ -371,15 +370,20 @@ class DebuggedApplication:
 
     def get_resource(self, request: Request, filename: str) -> Response:
         """Return a static resource from the shared folder."""
-        filename = join("shared", basename(filename))
+        path = join("shared", basename(filename))
+
         try:
-            data = pkgutil.get_data(__package__, filename)
+            data = pkgutil.get_data(__package__, path)
         except OSError:
-            data = None
-        if data is not None:
-            mimetype = mimetypes.guess_type(filename)[0] or "application/octet-stream"
-            return Response(data, mimetype=mimetype)
-        return Response("Not Found", status=404)
+            return NotFound()  # type: ignore[return-value]
+        else:
+            if data is None:
+                return NotFound()  # type: ignore[return-value]
+
+            etag = str(adler32(data) & 0xFFFFFFFF)
+            return send_file(
+                BytesIO(data), request.environ, download_name=filename, etag=etag
+            )
 
     def check_pin_trust(self, environ: "WSGIEnvironment") -> t.Optional[bool]:
         """Checks if the request passed the pin test.  This returns `True` if the
